@@ -1,12 +1,14 @@
 /*
  Copyright (C) 1997, Victor Edneral, e-mail: edneral@theory.npi.msu.su
+ Corrected by A.Pukhov 
 */
+
 #include "syst.h"
 #include "crt.h"
 #include <math.h>
 #include"crt0.h"
-int texflag=FALSE;
-int blind = FALSE;
+int texflag=0;
+int blind = 1;
 char    boxFrame[9] = {1,2,3,4,5,6,7,8,9};
 int nColors=15;
 int fColor=White;
@@ -42,7 +44,7 @@ static graphComPtr graphList=NULL;
 
 static void print0(int xc,int yc, int color,int bkcolor, char * s);
 
-static void texline(int x1,int y1,int x2,int y2, int arrowflag)
+static void texline(int x1,int y1,int x2,int y2, int arrowflag) /* by A.Pukhov */
 { struct tg_linesettingstype ls;
   double x,y,fx,fy;
   double dashsiz=3, dotsiz=1;
@@ -105,8 +107,21 @@ static void texline(int x1,int y1,int x2,int y2, int arrowflag)
    }
 }
 
+static void texBar(int x1,int y1,int x2,int y2) /* by A.Pukhov */
+{ struct tg_linesettingstype ls;
+  double x,y,fx,fy;
 
-static void texouttext(int x,int y,char* txt)
+   tg_getlinesettings(&ls);
+   x = (x1+texxshift)*texxscale;
+   y = (texymax1-y1)*texyscale;
+   fx = (x2+texxshift)*texxscale;
+   fy = (texymax1-y2)*texyscale;
+   f_printf(out_tex,"\\BBox(%.1f,%.1f)(%.1f,%.1f) \n" ,x,y,fx,fy);
+}
+
+
+
+static void texouttext(int x,int y,char* txt)   /* by A.Pukhov */
 {  struct tg_textsettingstype ts;
    f_printf(out_tex,"\\Text(%.1f,%.1f)[",(x+texxshift)*texxscale,
         (texymax1-y)*texyscale);
@@ -126,7 +141,7 @@ static void texouttext(int x,int y,char* txt)
 }
 
 
-int setTexCharSize(char * font)
+int setTexCharSize(char * font)   /* by A.Pukhov */
 {
         if(strcmp(font,"tiny") ==0 ) texCharScale=0.672;
    else if(strcmp(font,"scriptsize") ==0 ) texCharScale=0.782;
@@ -185,7 +200,7 @@ void	tg_setlinestyle(int linestyle, int thickness)
 }
 
 static void   addToGraphList(int type,int x1,int y1,int x2,int y2,
-int color,int aux,char*text)
+int color,int aux,char*text)             /* by A.Pukhov */
 { graphComPtr p=graphList ,pred=NULL;
 
   while(p)
@@ -214,12 +229,12 @@ int color,int aux,char*text)
      {
 	 if(pred==NULL)
 	 {  graphList=p->next;
-	    if (p->type==string) free(p->text);
+	    if (p->text) free(p->text);
             free(p);
             p=graphList;
          }else
 	 {  pred->next=p->next;
-	    if (p->type==string) free(p->text);
+	    if (p->text) free(p->text);
             free(p);
             p=pred->next;
          }
@@ -245,14 +260,14 @@ int color,int aux,char*text)
   if(pred==NULL) graphList=p; else pred->next=p;
 }
 
-static void repeatGrCom(int x1,int y1,int x2,int y2,int all)
+static void repeatGrCom(int x1,int y1,int x2,int y2,int all) /* by A.Pukhov */
 {  graphComPtr p = graphList;
    graphComPtr pred = NULL;
    graphComPtr p1,p2;
    int dn,xx1,xx2,yy1,yy2;
    int count =0;
 
-   if(blind) return;
+   if(blind==1) return;
 
 /*   printf(" repeatGrCom   "); */
    sg_screenSize(&bgi_maxx,&bgi_maxy);
@@ -335,16 +350,19 @@ contin:;
 
 
 
-static void crt_expose(int x1,int y1,int w,int h)
+static void crt_expose(int x1,int y1,int w,int h)  /* by A.Pukhov */
 {   
-    repeatGrCom(x1,y1,x1+w,y1+h,TRUE);
+    repeatGrCom(x1,y1,x1+w,y1+h,1);
     refresh_scr();     
 }
 
 void tg_bar(int left,  int top, int right, int bottom)
 { int x1,x2,y1,y2;
-
-  if(!blind && !texflag)
+  if(texflag) 
+  {
+    texBar(left,top,right,bottom);
+  }
+  else if(blind!=1)
   {
     x1=left +bgi_vport.left;
     y1=top+bgi_vport.top;
@@ -364,13 +382,14 @@ void  tg_clearviewport(void)
 
 void  tg_line(int x1, int y1, int x2, int y2)
 {
-  if(texflag) texline(x1,y1,x2,y2,0); else
+  if(texflag) texline(x1,y1,x2,y2,0); 
+  else
   {
      x1+=bgi_vport.left;
      y1+=bgi_vport.top;
      x2+=bgi_vport.left;
      y2+=bgi_vport.top;
-     if(!blind)
+     if(blind!=1)
      { sg_drawLine(x1,y1,x2,y2,fColor,bgi_line.thickness,bgi_line.linestyle);
        addToGraphList(line,x1,y1,x2,y2,fColor,
 		 bgi_line.thickness+16*bgi_line.linestyle,NULL);
@@ -448,8 +467,8 @@ int tg_textwidth(char* textstring)
     }
     return (int)(sum*texCharScale/texxscale);
   }
-  if(!blind) sg_textSize(textstring , &dx, &dy);else
-  dx=strlen(textstring)*10;
+  if(blind!=1) sg_textSize(textstring , &dx, &dy);
+  else       dx=strlen(textstring)*10;
   return  dx;
 }
 
@@ -457,25 +476,23 @@ int tg_textheight(char* textstring)
 {
   int dx,dy;
   if(texflag) return (int)(10*texCharScale/texyscale);
-  if(!blind) sg_textSize(textstring , &dx, &dy);else dy=8;
+  if(blind!=1) sg_textSize(textstring , &dx, &dy); else dy=8;
   return  dy;
 }
 
-void	 tg_outtextxy( int x, int y, char* textstring)
+void  tg_outtextxy( int x, int y, char* textstring)
 { int dx,dy;
 
   if(texflag) texouttext(x,y,textstring); else
   {  x+=bgi_vport.left;
      y+=bgi_vport.top;
-     if(!blind) sg_textSize(textstring,&dx,&dy);
-
-     x-=(dx*bgi_hj)/2 ;
-     y+=(dy* bgi_vj)/2 ;
-     if(!blind)
-     {  sg_outText(x,y,fColor,textstring);
+     if(blind!=1) 
+     {  sg_textSize(textstring,&dx,&dy);
+        x-=(dx*bgi_hj)/2 ;
+        y+=(dy* bgi_vj)/2 ;
+        sg_outText(x,y,fColor,textstring);
 	addToGraphList(string,x,y,x+dx-1,y-dy+1,fColor,0,textstring);
      }
-
   }
 }
 
@@ -503,33 +520,40 @@ void	 tg_rectangle( int left, int top, int right, int bottom )
 
 void tg_getlinesettings(struct tg_linesettingstype *  sls)
 {
-sls->linestyle=bgi_line.linestyle;
-sls->thickness=bgi_line.thickness;
+   sls->linestyle=bgi_line.linestyle;
+   sls->thickness=bgi_line.thickness;
 }
 
-void refresh_scr(void)
-{
-if(!blind) crt0_keypressed();
+void refresh_scr(void) { if(blind!=1) crt0_keypressed(); }
 
-}
-
-void start1(char * window_name,char * icon_file,char * ini_file)
-{
-   if(!blind)
-   {  crt0_start(window_name,icon_file,ini_file,&nColors);
+int  start1(char * window_name,char * icon_file,char * ini_file, 
+   void(*xw_errorExit)(void))   /* by A.Pukhov */
+{  int err=0;
+   if(blind!=1)
+   { err=crt0_start(window_name,icon_file,ini_file,&nColors,xw_errorExit);
       xw_expose=(*crt_expose);
       crt0_charSize(&CdX,&CdY);
       clr_scr(FGmain,BGmain);
+      if(err==1)
+      {  printf("You have launched CalcHEP compiled without X11 (/usr/include/X11 is empty)\n"
+                "Current version can work only in the 'blind' mode. ( See manual)\n"
+                "To use CalcHEP in graphic interface mode one has to install\n"
+                "   libX11-devel   for Fedora/Scientific, Darwin(MAC)\n"
+                "   libX11-dev     for Ubuntu/Debian\n"
+                "   xorg-x11-devel for SUSE\n"
+                "and recompile CalcHEP by\n"
+                "   make clean\n"
+                "   make\n");
+         exit(1);       
+      }    
    }
+   if(blind==2) printf("\n\"");
+   return err;
 }
 
-void finish(void){ if(!blind) crt0_finish(); }
+void finish(void){ if(blind!=1) crt0_finish(); if(blind==2) printf("\"\n");}
 
-void goto_xy(int x,int y)
-{
-   xc = x-1;
-   yc = y-1;
-}
+void goto_xy(int x,int y) {  xc = x-1; yc = y-1; }
 
 void clr_scr(int color, int bkcolor)
 {
@@ -545,7 +569,7 @@ void clr_scr(int color, int bkcolor)
    scrcolor(color,bkcolor);
    tg_setlinestyle(SolidLn,NormWidth);
    tg_settextjustify(LeftText,TopText);
-   if(!blind)
+   if(blind!=1)
    {
      sg_screenSize(&bgi_maxx,&bgi_maxy);
      tg_setviewport(0,0,bgi_maxx,bgi_maxy);
@@ -563,7 +587,7 @@ int where_y(void) {  return (yc + 1); }
 
 void clr_eol(void) { clrbox(xc+1,yc+1,maxCol(),yc+1); }
 
-static void print0(int xc,int yc, int color,int bkcolor, char * s)
+static void print0(int xc,int yc, int color,int bkcolor, char * s) /* by A.Pukhov*/
 {char buff[STRSIZ];
  int i,x,y,h1,w1;
  if (xc/CdX>maxCol())return;
@@ -612,7 +636,7 @@ static void print0(int xc,int yc, int color,int bkcolor, char * s)
   }
 }
 
-void get_text(int x1,int y1,int x2,int y2,void ** dump)
+void get_text(int x1,int y1,int x2,int y2,void ** dump) /* by A.pukhov */
 {
  int dn;
  graphComPtr p,q;
@@ -665,13 +689,13 @@ void get_text(int x1,int y1,int x2,int y2,void ** dump)
 /* printf(" OK\n"); */
 }
 
-void put_text(void ** dump)
+void put_text(void ** dump)  /* by A.Pukhov */
 {
    graphComPtr p,q;
 
    p= (graphComPtr)(*dump);
    if(p==NULL) return;
-   repeatGrCom(p->x1,p->y1,p->x2,p->y2,FALSE);
+   repeatGrCom(p->x1,p->y1,p->x2,p->y2,0);
 
    q=p;
    p=p->next;
@@ -701,7 +725,7 @@ void del_text(void ** dump)
    }
 }
 
-int print(char * format, ...)
+int print(char * format, ...)  /* by A.Pukhov */
 {  va_list args;
    char dump[STRSIZ], *d;
    va_start(args, format);
@@ -717,7 +741,7 @@ int print(char * format, ...)
       q = (int) strlen(d);
       if(q)
       {
-         if(!blind)
+         if(blind!=1)
          {
            print0(xc*CdX,yc*CdY,fColor,bColor,d);
            addToGraphList(t_xt,xc*CdX,yc*CdY,(xc+q)*CdX-1,(yc+1)*CdY-1,fColor,bColor,d);
@@ -748,12 +772,12 @@ void scrcolor(int f_col,int b_col)
 
 void clrbox(int x1,int y1,int x2,int y2)
 {
-  repeatGrCom((x1-1)*CdX,(y1-1)*CdY,x2*CdX-1,y2*CdY-1,FALSE);
+  repeatGrCom((x1-1)*CdX,(y1-1)*CdY,x2*CdX-1,y2*CdY-1,0);
   xc = x1-1;
   yc = y1-1;
 }
 
-/* LaTeX */
+/* LaTeX   by A.Pukhov  */
 double texX(double x)
 {
   return (x+texxshift)*texxscale;
@@ -766,49 +790,63 @@ double texY(double y)
 
 
 /* Sound support */
-void be_be(void){if(!blind) crt0_beep();}
+void be_be(void){ if(blind!=1) crt0_beep();}
 
 void clearTypeAhead(void)
-{  
-	 if (blind) return;
-	 while (crt0_keypressed()) crt0_inkey();
-}
+{  if(blind!=1) while (crt0_keypressed()) crt0_inkey(); }
 
 
-int  inkey(void)
-{ int      key; 
-  if(!inkeyString) do {key = crt0_inkey();} 
-                           while ((key == KB_MOUSE)&&mouse_info.but1!=2); 
+int  inkey(void)   /* by A.Pukhov */
+{ int  key; 
+
+  if(!inkeyString) for(;;) 
+  {    
+     key = crt0_inkey(); 
+     if(blind==2) 
+     { 
+       if(strchr("{}[]\\", key) || key==KB_MOUSE ) continue; 
+       if(key==KB_ESC)        { printf("}"); return key;}
+       else if(key==KB_DOWN)  { printf("["); return key;}
+       else if(key==KB_UP)    { printf("]"); return key;}
+       else if(key==KB_ENTER) { printf("{"); return key;}
+       else if(isprint(key))  { printf("%c",key); return key;}
+       else { printf("\\%.2x",key); return key;}
+       continue;
+     }
+      
+     if((key == KB_MOUSE)&&mouse_info.but1!=2) continue;  
+     break;
+  } 
   else      
   { 
     key= inkeyString[0];  
-    if(key==0) {if(blind) exit(101); else inkeyString=NULL;}
+    if(key==0) { if(blind==1) sortie(101); else inkeyString=NULL; }
     else   inkeyString++;   
   } 
 
   if(inkeyString)
-  {
+  { 
     switch (key)
     { 
-      case '{':   key= KB_ESC ;break;
-      case ']':   key=KB_RIGHT;break;
-      case '[':   key=KB_LEFT;break;
-      case '}':   key=KB_ENTER;break;
+      case '}':   key=KB_ESC ;  break;
+      case '[':   key=KB_DOWN;  break;
+      case ']':   key=KB_UP;    break;
+      case '{':   key=KB_ENTER; break;
+      case '\\':   sscanf(inkeyString,"%2x",&key); inkeyString+=2; break;
     }
   }
+  
 /*
-if(blind)
+if(blind==1)
 {
 if (key>30)  f_printf(stderr,"inkey='%c'\n",key);
  else         f_printf(stderr,"inkey=%d\n",key);
 }
 */
-
   return key;
-}   /*  inkey0  */
+}
 
-
-int  escpressed(void)
+int  escpressed(void)  /* by A.Pukhov */
 {  int key;
    if(blind) return 0;
    if(crt0_keypressed()) key=crt0_inkey();else return 0;
@@ -818,4 +856,3 @@ int  escpressed(void)
      if(crt0_keypressed())  key=crt0_inkey(); else return 0;
    }
 }
-
