@@ -1,7 +1,9 @@
 #ifndef __Q_KIN__
 #define __Q_KIN__
 
-extern int mkmom(double *x, double *tfact);
+#include"nType.h"
+
+extern int mkmom(double *x, double *tfact,REAL*pv);
 extern int imkmom(double P1,double P2);
 
 #endif
