@@ -20,8 +20,9 @@
 #include<stdlib.h>
 #include <memory.h>
 #include <string.h>
-#endif
 
+#endif
+#include "syst.h"
 #include "colorf.h"
 
 int MAXGLEN, SIZE, MAX_POW;
@@ -32,8 +33,8 @@ static int cerror(int n,char* s)
 { /*  - generate error message occur in color package - 08/01/90  */
 
    fprintf(stderr,"***** %s\n",s);
-        if (n > CERRLEV1) fprintf(stderr,"Runtime error %u\n",n), exit(1313);
-   else if (n > CERRLEV2) exit(99); 
+        if (n > CERRLEV1) fprintf(stderr,"Error  %u in  colorf.c \n",n), sortie(55);
+   else if (n > CERRLEV2) sortie(56); 
    return 0;
 }  
 
@@ -153,6 +154,7 @@ static int getv(cgraph* cg)
 
    if (n > MAXGLEN) cerror(254,"GetV: no free vertex in C-graph"); 
    else  { ++(cg->gl);  return n; }  
+   return 0;
 }  
 
 
@@ -280,6 +282,7 @@ static void remqg_qg(int n0,int n1, cgraph ** c)
 static void rev3g(int en,cvertex* v)
 { /* - reverse 3G vertex such that edge EN will be first - 08/01/90  */  
    if (en != v->e[0]) 
+   {
       if (en == v->e[1]) 
       { 
          v->e[1] = v->e[2]; 
@@ -292,7 +295,8 @@ static void rev3g(int en,cvertex* v)
             v->e[1] = v->e[0]; 
             v->e[0] = en; 
          }  
-         else  cerror(255,"Rev3G: Invalid select edge"); 
+         else  cerror(255,"Rev3G: Invalid select edge");
+   }      
 }  
 
 
@@ -582,17 +586,17 @@ void fct_print(factor *fct, char *s)
   for(i=0;i<fct->len;i++)  
 	  if(fct->nc[i]) 
 	  {  
-		  sprintf(s+strlen(s),"%+d",fct->nc[i]);
+		  sprintf(s+strlen(s),"%+ld",fct->nc[i]);
 		  if(i) sprintf(s+strlen(s),"N^%d",i);
 	  }
   sprintf(s+strlen(s),")");
 
   if(fct->dpow>0) 
-	  sprintf(s+strlen(s),"/(%d*N^%d)",fct->dc,fct->dpow);
+	  sprintf(s+strlen(s),"/(%ld*N^%d)",fct->dc,fct->dpow);
   else if(fct->dpow<0) 
-	  sprintf(s+strlen(s),"*N^%d/%d",-fct->dpow,fct->dc);
+	  sprintf(s+strlen(s),"*N^%d/%ld",-fct->dpow,fct->dc);
   else
-	  sprintf(s+strlen(s),"/%d",fct->dc);
+	  sprintf(s+strlen(s),"/%ld",fct->dc);
 }
 
   /* ************************** Cross reference ************************* */ 
@@ -637,3 +641,65 @@ void fct_print(factor *fct, char *s)
   /* *            +------> RedND                                        * */ 
   /* *                                                                  * */ 
   /* ******************************************************************** */
+
+vtype typev(vert0 v,int valence)
+{int  ng = 0, ne, nq = 0;
+ 
+/* Return color type of vertex - 06/01/90  */ 
+      
+   for (ne = 0; ne < valence; ne++)
+      if (prtclbase[v[ne].partcl-1].cdim != 1) 
+      {   if (prtclbase[v[ne].partcl-1].cdim == 8) ng++; else nq++;} 
+   switch (ng) 
+   {    
+      case 0:   return nq == 2 ? tv : zv; 
+      case 1:   return qg; 
+      case 2:   return g2; 
+      case 3:   return g3; 
+      default:  return cerror(252,"TypeV: invalid vertex type"); 
+   }  /* case */ 
+}  /* TypeV */ 
+
+
+void t2k2(vcsect* g, int * nv, cvertex * vl)
+{
+  int   i,k, ne, en=0, maxnv=*nv; 
+  int   maptar[2*maxvert][MAXVALENCE];
+    
+/* Transfer Taranov's representation of graph to Kryukov's representation  */ 
+/* - 07/01/90  */ 
+    
+   for(i=0;i<2*maxvert;i++) for(k=0;k<MAXVALENCE;k++) maptar[i][k]=0;
+   *nv=0;
+   for(i=0;i<g->sizet; i++)  if (typev(g->vertlist[i],g->valence[i]) != zv) 
+   { 
+      if(*nv>= maxnv) cerror(251,"To many vertices");
+      vl[*nv].e[0]=0; vl[*nv].e[1]=0;  vl[*nv].e[2]=0;
+      vl[*nv].vt = typev(g->vertlist[i],g->valence[i]); 
+
+      for (k=0, ne=0; ne<g->valence[i] ;ne++)
+      {  int dim= prtclbase[g->vertlist[i][ne].partcl-1].cdim;
+         if(dim!=1)
+         {  int l=maptar[i][ne];
+            if(!l)
+            { 
+               l = ++en; 
+               maptar[i][ne] = l;
+               maptar[g->vertlist[i][ne].link.vno]
+                     [g->vertlist[i][ne].link.edno]=l; 
+            } 
+         
+            if (vl[*nv].vt != g3 && vl[*nv].vt != g2) 
+            switch (dim) 
+            {
+              case  8:  vl[*nv].e[0]=l;  break; 
+              case -3:  vl[*nv].e[1]=l;  break; 
+              case  3:  vl[*nv].e[2]=l;  break; 
+              default: cerror(252,"t2k - invalid particle color");
+            } else vl[*nv].e[k++] = l;
+         } 
+      }  
+      (*nv)++;
+   } 
+}
+
