@@ -5,7 +5,7 @@
 typedef struct
 {
   int power;
-  short mass,width;
+  int mass,width;
   char momStr[MAXINOUT];
 } denom_struct;
       
