@@ -5,29 +5,31 @@
 #include"model.h"
 #include"physics.h"
 
-#define whohowMAX 100
+#define whohowMAX 300
 typedef struct whohow
 {  int    who, how;
 }  whohow[whohowMAX];
 
 typedef struct hadron
 {
-   char        name[8];
+   char        name[P_NAME_SIZE];
    shortstr    contents;
    int         pow;
-   int         parton[100];
+   int         parton[1000];
+   int         polarized[1000];
 }  hadron;
 
-extern whohow     liminsp;
+extern whohow     liminsp, LimQ;
 extern whohow     limout;
 extern void  nilprtcl(whohow      p_list);
 
 
-extern shortstr processch, limpch,deloutch;
+extern shortstr processch;
+char limpch[STRSIZ], deloutch[STRSIZ];
 extern int  nin, nout, n_x;   /* Number of X-particles */
 extern int  enter(void);
 extern  hadron hadrons[MAXINOUT];
 
-
+extern int polarized(int p, int Prtcl);
 
 #endif
