@@ -8,8 +8,8 @@
 
 extern  denom_struct   denom[2 * maxvert - 2];
 
-extern byte   denrno;
+extern int  denrno;
 
-extern void  saveanaliticresult(poly rnum, poly factn, poly factd, vcsect vcs);
+extern void  saveanaliticresult(poly rnum, poly factn, poly factd, vcsect vcs, int nFile);
 
 #endif
